@@ -80,6 +80,19 @@ use serde::de::{Error, Visitor};
 #[cfg(feature = "serde")]
 use serde::{Deserialize, Deserializer, Serialize, Serializer};
 
+// Trick to test README samples (from: https://github.com/rust-lang/cargo/issues/383#issuecomment-720873790)
+#[cfg(doctest)]
+mod test_readme {
+    macro_rules! external_doc_test {
+        ($x:expr) => {
+            #[doc = $x]
+            extern "C" {}
+        };
+    }
+
+    external_doc_test!(include_str!("../README.md"));
+}
+
 #[derive(Clone, Debug)]
 enum FlexStrInner<const N: usize, T> {
     /// A wrapped string literal
